@@ -1,3 +1,4 @@
+import copy
 from typing import *
 
 from cognite.client import utils
@@ -13,6 +14,7 @@ class RelationshipsAPI(APIClient):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._CREATE_LIMIT = 1000
+        self._LIST_SUBQUERY_LIMIT = 100
 
     def _create_filter(
         self,
@@ -60,6 +62,7 @@ class RelationshipsAPI(APIClient):
         active_at_time: Dict[str, int] = None,
         labels: LabelFilter = None,
         limit: int = None,
+        fetch_resources: bool = False,
     ) -> Generator[Union[Relationship, RelationshipList], None, None]:
         """Iterate over relationships
 
@@ -100,7 +103,19 @@ class RelationshipsAPI(APIClient):
             active_at_time=active_at_time,
             labels=labels,
         )
-        return self._list_generator(method="POST", limit=limit, filter=filter)
+        if (
+            len(filter.get("targetExternalIds", [])) > self._LIST_SUBQUERY_LIMIT
+            or len(filter.get("sourceExternalIds", [])) > self._LIST_SUBQUERY_LIMIT
+        ):
+            raise ValueError(
+                "For queries with more than {} source_external_ids or target_external_ids, only list is supported".format(
+                    self._LIST_SUBQUERY_LIMIT
+                )
+            )
+
+        return self._list_generator(
+            method="POST", limit=limit, filter=filter, other_params={"fetchResources": fetch_resources}
+        )
 
     def __iter__(self) -> Generator[Relationship, None, None]:
         """Iterate over relationships
@@ -112,30 +127,36 @@ class RelationshipsAPI(APIClient):
         """
         return self.__call__()
 
-    def retrieve(self, external_id: str) -> Optional[Relationship]:
+    def retrieve(self, external_id: str, fetch_resources: bool = False) -> Optional[Relationship]:
         """Retrieve a single relationship by external id.
 
         Args:
             external_id (str): External ID
+            fetch_resources (bool): if true, will try to return the full resources referenced by the relationship in the
+                source and target fields.
 
         Returns:
             Optional[Relationship]: Requested relationship or None if it does not exist.
 
         Examples:
 
-            Get relationship by external id::
+            Get relationship by external id:
 
                 >>> from cognite.client.beta import CogniteClient
                 >>> c = CogniteClient()
                 >>> res = c.relationships.retrieve(external_id="1")
         """
-        return self._retrieve_multiple(external_ids=external_id, wrap_ids=True)
+        return self._retrieve_multiple(
+            external_ids=external_id, wrap_ids=True, other_params={"fetchResources": fetch_resources}
+        )
 
-    def retrieve_multiple(self, external_ids: List[str]) -> RelationshipList:
+    def retrieve_multiple(self, external_ids: List[str], fetch_resources: bool = False) -> RelationshipList:
         """Retrieve multiple relationships by external id.
 
         Args:
             external_ids (List[str]): External IDs
+            fetch_resources (bool): if true, will try to return the full resources referenced by the relationship in the
+                source and target fields.
 
         Returns:
             RelationshipList: The requested relationships.
@@ -149,7 +170,9 @@ class RelationshipsAPI(APIClient):
                 >>> res = c.relationships.retrieve_multiple(external_ids=["abc", "def"])
         """
         utils._auxiliary.assert_type(external_ids, "external_id", [List], allow_none=False)
-        return self._retrieve_multiple(external_ids=external_ids, wrap_ids=True)
+        return self._retrieve_multiple(
+            external_ids=external_ids, wrap_ids=True, other_params={"fetchResources": fetch_resources}
+        )
 
     def list(
         self,
@@ -167,6 +190,7 @@ class RelationshipsAPI(APIClient):
         active_at_time: Dict[str, int] = None,
         labels: LabelFilter = None,
         limit: int = 100,
+        fetch_resources: bool = False,
     ) -> RelationshipList:
         """Lists relationships stored in the project based on a query filter given in the payload of this request. Up to 1000 relationships can be retrieved in one operation.
 
@@ -186,6 +210,8 @@ class RelationshipsAPI(APIClient):
             labels (LabelFilter): Return only the resource matching the specified label constraints.
             limit (int): Maximum number of relationships to return. Defaults to 100. Set to -1, float("inf") or None
                 to return all items.
+            fetch_resources (bool): if true, will try to return the full resources referenced by the relationship in the
+                source and target fields.
 
         Returns:
             RelationshipList: List of requested relationships
@@ -223,7 +249,37 @@ class RelationshipsAPI(APIClient):
             active_at_time=active_at_time,
             labels=labels,
         )
-        return self._list(method="POST", limit=limit, filter=filter)
+        target_external_ids = filter.get("targetExternalIds", [])
+        source_external_ids = filter.get("sourceExternalIds", [])
+        if len(target_external_ids) > self._LIST_SUBQUERY_LIMIT or len(source_external_ids) > self._LIST_SUBQUERY_LIMIT:
+            if limit not in [-1, None, float("inf")]:
+                raise ValueError(
+                    "Querying more than {} source_external_ids/target_external_ids only supported for queries without limit (pass -1 / None / inf instead of {}".format(
+                        self._LIST_SUBQUERY_LIMIT, limit
+                    )
+                )
+            tasks = []
+
+            for ti in range(0, max(1, len(target_external_ids)), self._LIST_SUBQUERY_LIMIT):
+                for si in range(0, max(1, len(source_external_ids)), self._LIST_SUBQUERY_LIMIT):
+                    task_filter = copy.copy(filter)
+                    if target_external_ids:  # keep null if it was
+                        task_filter["targetExternalIds"] = target_external_ids[ti : ti + self._LIST_SUBQUERY_LIMIT]
+                    if source_external_ids:  # keep null if it was
+                        task_filter["sourceExternalIds"] = source_external_ids[si : si + self._LIST_SUBQUERY_LIMIT]
+                    tasks.append((task_filter,))
+
+            tasks_summary = utils._concurrency.execute_tasks_concurrently(
+                lambda filter: self._list(
+                    method="POST", limit=limit, filter=filter, other_params={"fetchResources": fetch_resources}
+                ),
+                tasks,
+                max_workers=self._config.max_workers,
+            )
+            tasks_summary.raise_compound_exception_if_failed_tasks()
+            rels = RelationshipList([rel for result in tasks_summary.joined_results() for rel in result])
+            return rels
+        return self._list(method="POST", limit=limit, filter=filter, other_params={"fetchResources": fetch_resources})
 
     def create(self, relationship: Union[Relationship, List[Relationship]]) -> Union[Relationship, RelationshipList]:
         """Create one or more relationships.
